#!/usr/bin/env node

'use strict';

const __root = process.cwd();
const fs = require('fs');
const path = require('path');
const program = require('commander');

const PROJECT_TEMPLATE_PATH = path.resolve(__dirname, './templates/project');
const COMPONENT_TEMPLATE_PATH = path.resolve(__dirname, './templates/component');
const COMPONENT_DESC_PATH = path.resolve(__dirname, './templates/description');
const COMPONENT_IMG_PATH = path.resolve(__dirname, './templates/images');

const COMPONENT_NAME_CATEGORY = '__component_category__';
const COMPONENT_NAME_TEMPLATE = '__component__';
const COMPONENT_NAME_TEMPLATE_CAMEL = '__component_camel__';
const COMPONENT_NAME_TEMPLATE_HYPHEN = '__component_hypen__';
const COMPONENT_NAME_TEMPLATE_CHINESE = '__component_chinese__';
const COMPONENT_NAME_TEMPLATE_LOWER_CASE = '__component_lower_case__';

const JS_IMPORT = ' 图表导入开始';
const JS_DEFINE = 'const components = \\[';
const JS_DATA_PATH = 'var componentJsonDataPaths = \\[';

var componentCategory = '';
var componentPascalName = '';
var componentCamelName = '';
var componentHyphenName = '';
var componentChineseName = '';

function initProj(projectName) {
    if (!projectName) {
        return;
    }
    console.log(`准备创建 AVW 二次开发组件项目：${projectName}...`);
    var projectPath = __root + '/' + projectName;
    if (!fs.existsSync(projectPath)) {
        fs.mkdirSync(projectPath);
        copyFolder(PROJECT_TEMPLATE_PATH, projectPath);
        console.log('信息: 已创建 AVW 二次开发组件项目。');
        // 创建成功以后，进入项目目录
        console.log('信息: 正在复制文件...');
        setTimeout(function () {
            console.log('信息: 项目创建成功，您还需要进入项目目录并运行 "npm install" 完成项目设置。');
            console.log(
                `
===================================   提示   ===================================
要启动 SDK，请进入项目目录，先运行 "npm install"，然后运行 "npm run serve"。
================================================================================
`
            );
        }, 1000);

    } else {
        console.error(`错误: ${projectName} 项目已经存在，请使用其他名称。`);
    }
}

// 创建新组件
function createComponent(componentsStr) {
    if (!componentsStr) {
        return;
    }
    let components = componentsStr.toString().split('/');
    for (let i = 0; i < components.length; i++) {
        components[i] = components[i].replace(/ /g, '');
    }
    if (!components || !components.length || components.length != 3) {
        console.error('错误: 请输入创建的组件类别、组件英文名称、组件中文名称，用 "/" 连接，如 "avwsdk -c categoryA/MyComponent/我的组件"。');
        return;
    }
    if (!/^[a-z][A-Za-z0-9]+$/.test(components[0])) {
        console.error('错误: 组件类别请使用驼峰命名法（首单词小写，之后单词首字母大写，单词与单词之间不加空格或连接符，如 "categoryA"），建议使用单个小写单词。');
        return;
    }
    if (!/^[A-Z][A-Za-z0-9]+$/.test(components[1])) {
        console.error('错误: 组件名称请使用 Pascal 命名法（每个单词首字母大写，单词与单词之间不加空格或连接符，如 "MyComponent"）。');
        return;
    }
    var componentCategoryPath = `${__root}/${components[0]}`;
    var componentPath = `${componentCategoryPath}/${pascalToCamelName(components[1])}`;
    if (!fs.existsSync(`${__root}/package.json`)) {
        console.error('错误: 请先创建项目（avwsdk -p <projectName>），然后确保在项目目录中运行 avwsdk -c <category>/<name>/<chineseName> 命令。');
        return;
    }
    // 如果不存在这个目录，创建之
    if (!fs.existsSync(componentCategoryPath)) {
        fs.mkdirSync(componentCategoryPath);
    }
    if (!fs.existsSync(componentPath)) {
        if (!/^[\u4e00-\u9fa5][\u4e00-\u9fa5A-Za-z0-9]+$/.test(components[2])) {
            console.error('警告: 组件中文名称未使用中文。出于用户体验原因，建议使用中文。这是非强制性的建议，您可以继续开发。');
        }
        fs.mkdirSync(componentPath);
        componentCategory = components[0]
        componentPascalName = components[1];
        componentCamelName = pascalToCamelName(componentPascalName);
        componentHyphenName = pascalToHyphenName(componentPascalName);
        componentChineseName = components[2];
        copyFolder(COMPONENT_TEMPLATE_PATH, componentPath);

        var componentPublicPath = `${__root}/public/${componentCategory}`;
        var componentDescPath = `${componentPublicPath}/description`;
        var componentImgPath = `${componentPublicPath}/images`;
        // 如果不存在这个目录，创建之
        if (!fs.existsSync(componentPublicPath)) {
            fs.mkdirSync(componentPublicPath);
        }
        if (!fs.existsSync(componentDescPath)) {
            fs.mkdirSync(componentDescPath);
        }
        if (!fs.existsSync(componentImgPath)) {
            fs.mkdirSync(componentImgPath);
        }
        copyFolder(COMPONENT_DESC_PATH, componentDescPath);
        copyFolder(COMPONENT_IMG_PATH, componentImgPath);
        // 设置 SDK 层级 index.js 和 data.js
        replaceFileSync(`${__root}/charts/index.js`, JS_IMPORT, `${JS_IMPORT}\nimport ${componentCamelName} from './${componentCategory}/${componentCamelName}/index';`);
        replaceFileSync(`${__root}/charts/index.js`, JS_DEFINE, `${JS_DEFINE.replace('\\[', '[')}\n    ${componentCamelName},`);
        replaceFileSync(`${__root}/charts/data.js`, JS_DATA_PATH, `${JS_DATA_PATH.replace('\\[', '[')}\n    './${componentCategory}/description/${componentChineseName}.json',`);
        console.warn(`信息: 新组件创建成功。`);
    }
    else {
        // 已经存在目录
        console.warn(`错误: ${components[1]} 已经存在，请使用其他名称。`);
        return;
    }
};

function copyFolder(src, dst) {
    let paths = fs.readdirSync(src); //同步读取当前目录
    paths.forEach(function (path) {
        var _src = src + '/' + path;
        var _dst = dst + '/' + path;
        fs.stat(_src, function (err, stats) {  //stats  该对象 包含文件属性
            if (err) throw err;
            if (stats.isFile()) { //如果是个文件则拷贝
                if (path !== '.DS_Store') {
                    // 忽略 .DS_Store
                    if (_dst.endsWith('component.vue')) {
                        _dst = _dst.replace(/component\.vue$/, `${componentPascalName}.vue`)
                    }
                    if (_dst.endsWith('component.png')) {
                        _dst = _dst.replace(/component\.png$/, `${componentPascalName.toLowerCase()}.png`)
                    }
                    if (_dst.endsWith('component.json')) {
                        _dst = _dst.replace(/component\.json$/, `${componentChineseName}.json`)
                    }
                    if (_dst.endsWith('package.template.json')) {
                        _dst = _dst.replace(/package\.template\.json$/, `package.json`)
                    }
                    let readable = fs.createReadStream(_src);//创建读取流
                    let writable = fs.createWriteStream(_dst);//创建写入流
                    readable.on('end', () => {
                        console.log(`信息: 已复制 ${_dst}`);
                        switch (path.toLowerCase()) {
                            case 'index.js':
                                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE, componentPascalName);
                                break;
                            case 'component.vue':
                                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_CAMEL, componentCamelName);
                                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_HYPHEN, componentHyphenName);
                                break;
                            case 'component.json':
                                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_CHINESE, componentChineseName);
                                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_HYPHEN, componentHyphenName);
                                replaceFileSync(_dst, COMPONENT_NAME_CATEGORY, componentCategory);
                                replaceFileSync(_dst, COMPONENT_NAME_TEMPLATE_LOWER_CASE, componentPascalName.toLowerCase());
                                break;
                            default:
                                break;
                        }
                    });
                    readable.pipe(writable);
                }

            } else if (stats.isDirectory()) { //是目录则 递归
                checkDirectory(_src, _dst, copyFolder);
            }
        });
    });
}

function checkDirectory(src, dst, callback) {
    fs.access(dst, fs.constants.F_OK, (err) => {
        if (err) {
            fs.mkdirSync(dst);
            callback(src, dst);
        } else {
            callback(src, dst);
        }
    });
}

function replaceFileSync(filePath, sourceRegx, targetStr) {
    let data = fs.readFileSync(filePath);
    let str = data.toString();
    str = str.replace(new RegExp(sourceRegx, 'gm'), targetStr);
    fs.writeFileSync(filePath, str);
}

function pascalToCamelName(name) {
    if (!name || name == '') {
        return 'invalidName';
    }
    return name.charAt(0).toString().toLowerCase() + name.substring(1, name.length);
}

function camelToHyphenName(name) {
    if (!name || name == '') {
        return 'invalid-name';
    }
    let newName = '';
    for (let i = 0; i < name.length; i++) {
        if (/[A-Z]/.test(name[i])) {
            newName += '-';
        }
        newName += name[i];
    }
    return newName.toLowerCase();
}

function pascalToHyphenName(name) {
    return camelToHyphenName(pascalToCamelName(name));
}

program
    .usage('[options] <names ...>')
    .option('-p, --newProject <projectName>', '创建新的 AVW 二次开发项目及其目录')
    .option('-c, --newComponent <category>/<name>/<chineseName>', '输入要创建的组件类别、名称、中文名称，组件类别采用驼峰命名法，名称采用 Pascal 命名法')
    .parse(process.argv);


initProj(program.newProject);
createComponent(program.newComponent);